import traceback
import json
import requests.utils
from requests.exceptions import RequestException


class ApiRequest(object):
    def __init__(self, logger, configuration, user_agent):
        self._logger = logger
        self._configuration = configuration
        self._user_agent = user_agent
        self.timeout = 10

    def post_request(self, endpoint, params=None, json_data=None):
        return self._request('POST', endpoint, params, json_data)

    def get_request(self, endpoint, params=None):
        return self._request('GET', endpoint, params)

    def put_request(self, endpoint, params=None, json_data=None):
        return self._request('PUT', endpoint, params, json_data)

    def delete_request(self, endpoint, params=None):
        return self._request('DELETE', endpoint, params)

    def _request(self, method, endpoint, params, json_data=None):
        try:
            headers = requests.utils.default_headers()
            headers.update({
                'Api-Token': self._configuration.api_token,
                'User-Agent': self._user_agent
            })
            response = requests.request(
                method, self._configuration.api_url + '/' + endpoint,
                params=params, headers=headers, json=json_data, timeout=self.timeout)
            response.raise_for_status()
            return json.loads(response.text)
        except RequestException as e:
            self._logger.error(
                "failed to %s request to url=%s, with params=%s. error is: %s" %
                (method, self._configuration.api_url, params, traceback.format_exc()))
            raise e
        except Exception as ex:
            self._logger.error(
                "unexpected Exception while trying to post request. error is: %s"
                % (traceback.format_exc()))
            raise ex



from activecampaign_api.api.abstract_request import AbstractRequest
from activecampaign_api.api.consts import API_ENDPOINT


class WebhookEvents(object):
    def __init__(self, api_request):
        self.api_request = api_request
        self.endpoint = API_ENDPOINT.WEBHOOK_EVENTS

    def get(self):
        return self.api_request.get_request(endpoint=self.endpoint)



from activecampaign_api.api.consts import API_ENDPOINT

ENDPOINT = API_ENDPOINT.SEGMENT

class Segment(object):
    def __init__(self, name=None, logic=None,
                 hidden=None, seriesid=None,
                 links=None, id=None):
        self._id = id
        self._name = name
        self._logic = logic
        self._hidden = hidden
        self._seriesid = seriesid
        self._links = links or []

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._name

    @property
    def logic(self):
        return self._logic

    @property
    def hidden(self):
        return self._hidden

    @property
    def seriesid(self):
        return self._seriesid

    @property
    def links(self):
        return self._links

    def from_dict(self, data):
        if 'id' in data:
            self._id = data['id']
        if 'name' in data:
            self._name = data['name']
        if 'logic' in data:
            self._logic = data['logic']
        if 'hidden' in data:
            self._hidden = data['hidden']
        if 'seriesid' in data:
            self._seriesid = data['seriesid']
        if 'links' in data:
            self._links = data['links']
        return self

    def to_dict(self):
        data = {
            'id': self._id,
            'name': self._name,
            'logic': self._logic,
            'hidden': self._hidden,
            'seriesid': self._seriesid,
            'links': self._links,
        }
        return data


class Segments(object):
    def __init__(self):
        self._segments = []
        self._meta = None

    def from_dict(self, data):
        if 'meta' in data:
            self._meta = data['meta']
        if 'segments' in data and isinstance(data['segments'], list):
            for segment_data in data['segments']:
                self._segments.append(Segment().from_dict(segment_data))
        return self

    @property
    def meta(self):
        return self._meta

    @property
    def segments(self):
        return self._segments


class SegmentRequest(object):
    def __init__(self, api_request):
        self.api_request = api_request

    def get_all(self):
        return Segments().from_dict(self.api_request.get_request(endpoint=ENDPOINT))

    def get(self, segment_id):
        return Segment.from_dict(self.api_request.get_request(endpoint=ENDPOINT + '/' + segment_id))




from activecampaign_api.api.abstract_request import AbstractRequest
from activecampaign_api.api.consts import API_ENDPOINT


class EmailActivitiesRequest(AbstractRequest):
    def __init__(self, api_request):
        super(EmailActivitiesRequest, self).__init__(api_request)
        self.endpoint = API_ENDPOINT.EMAIL_ACTIVITIES

    def get(self, params=None):
        return self.api_request.get_request(endpoint=self.endpoint, params=params)



import logging

from activecampaign_api.api.configuration import Configuration
from activecampaign_api.api.consts import USER_AGENT
from activecampaign_api.api.api_request import ApiRequest
from activecampaign_api.api.message import MessageRequest
from activecampaign_api.api.contact import ContactRequest
from activecampaign_api.api.segment import SegmentRequest
from activecampaign_api.api.custom_field import CustomFieldsRequest
from activecampaign_api.api.custom_field_value import CustomFieldValueRequest
from activecampaign_api.api.email_activities import EmailActivitiesRequest
from activecampaign_api.api.webhook import WebHook
from activecampaign_api.api.webhook_events import WebhookEvents


class Api(object):
    def __init__(self, api_url, api_token):
        self._logger = logging.getLogger('activecampaign_api')
        self._api_request = ApiRequest(self._logger, Configuration(api_url, api_token), USER_AGENT)

    @property
    def message(self):
        return MessageRequest(self._api_request)

    @property
    def contact(self):
        return ContactRequest(self._api_request)

    @property
    def segment(self):
        return SegmentRequest(self._api_request)

    @property
    def custom_field(self):
        return CustomFieldsRequest(self._api_request)

    @property
    def custom_field_value(self):
        return CustomFieldValueRequest(self._api_request)

    @property
    def email_activities(self):
        return EmailActivitiesRequest(self._api_request)

    @property
    def webhook(self):
        return WebHook(self._api_request)

    @property
    def webhook_events(self):
        return WebhookEvents(self._api_request)
